/*
 *  RapidMiner
 *
 *  Copyright (C) 2001-2012 by Rapid-I and the contributors
 *
 *  Complete list of developers available at our web site:
 *
 *       http://rapid-i.com
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package com.rapidminer.operator.nio.file;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import com.rapidminer.operator.OperatorException;
import com.rapidminer.repository.BlobEntry;
import com.rapidminer.repository.RepositoryException;
import com.rapidminer.repository.RepositoryLocation;
import com.rapidminer.tools.Tools;

/**
 * 
 * Simple implementation of a {@link FileObject} backed by a
 * {@link RepositoryLocation}. The repository entry has to be of type 'blob'.
 * 
 * @author Nils Woehler
 * 
 */
public class RepositoryBlobObject extends FileObject {

	private static final long serialVersionUID = 1L;

	private RepositoryLocation location;
	private File file = null;

	public RepositoryBlobObject(RepositoryLocation location) {
		super();
		this.location = location;
	}

	@Override
	public InputStream openStream() throws OperatorException {
		BlobEntry entry;
		try {
			if( location.locateEntry() instanceof BlobEntry){				
				entry = (BlobEntry) location.locateEntry();
				return entry.openInputStream();
			} else {
				throw new OperatorException("942", null, location.getPath(),"blob",location.locateEntry().getType());
			}
		} catch (RepositoryException e) {
			throw new OperatorException("319", e, location.getPath());
		}

	}

	@Override
	public File getFile() throws OperatorException {
		if (file == null) {
			try {
				file = File.createTempFile("rm_file_", ".dump");
				FileOutputStream fos = new FileOutputStream(file);
				BlobEntry entry;
				if( location.locateEntry() instanceof BlobEntry){				
					entry = (BlobEntry) location.locateEntry();
				} else {
					throw new OperatorException("942", null, location.getPath(),"blob",location.locateEntry().getType());
				}
				InputStream in = entry.openInputStream();
				Tools.copyStreamSynchronously(in, fos, true);
				file.deleteOnExit();
			} catch (IOException e) {
				throw new OperatorException("303", e, file, e.getMessage());
			} catch (RepositoryException e) {
				throw new OperatorException("319", e, location.getPath());
			}
			return file;
		} else {
			return file;
		}
	}

	@Override
	public String toString() {
		return (file != null) ? ("Repository location stored in temporary file: " + file
				.getAbsolutePath()) : ("Repository location: " + location
				.getAbsoluteLocation());
	}

	@Override
	protected void finalize() throws Throwable {
		if (file != null) {
			file.delete();
		}
		super.finalize();
	}

}
